@@ -1,5 +1,5 @@
 /*
-Copyright (C) 2012-2013 Andreas Schiffler
+Copyright (C) 2012-2014 Andreas Schiffler
 
 This software is provided 'as-is', without any express or implied
 warranty.  In no event will the authors be held liable for any damages
@@ -17,12 +17,12 @@ freely.
 #include <math.h>
 #include <time.h>
 
-#include "common.h"
+#include "SDL_test_common.h"
 
 #include "SDL2_gfxPrimitives.h"
 #include "SDL2_rotozoom.h"
 
-static CommonState *state;
+static SDLTest_CommonState *state;
 
 /* Screen size */
 #define WIDTH	DEFAULT_WINDOW_WIDTH
@@ -59,13 +59,13 @@ void RotatePicture (SDL_Surface *picture, int rotate, int flip, int smooth, int
 	SDL_Renderer *renderer = state->renderers[0];
 	SDL_Event event;
 
-	printf("%s\n", messageText);
+	SDL_Log("%s\n", messageText);
 
 	/* Rotate and display the picture */
 	framemax=4*360; 
 	frameinc=1;
 	for (framecount=-360; framecount<framemax && !done; framecount += frameinc) {
-		while (SDL_PollEvent(&event)) CommonEvent(state, &event, &done);
+		while (SDL_PollEvent(&event)) SDLTest_CommonEvent(state, &event, &done);
 		if ((framecount % 360)==0) frameinc++;
 		SDL_SetRenderDrawColor(renderer, 0xA0, 0xA0, 0xA0, 0xFF);
 		SDL_RenderClear(renderer);
@@ -87,7 +87,7 @@ void RotatePicture (SDL_Surface *picture, int rotate, int flip, int smooth, int
 			}
 			angle=framecount*rotate;
 			if (((framecount % 120)==0) || (delay>0)) {
-				printf ("  Frame: %i   Rotate: angle=%.2f  Zoom: x=%.2f y=%.2f\n",framecount,angle,zoomfx,zoomfy);
+				SDL_Log("  Frame: %i   Rotate: angle=%.2f  Zoom: x=%.2f y=%.2f\n",framecount,angle,zoomfx,zoomfy);
 			}
 			if ((rotozoom_picture=rotozoomSurfaceXY (picture, angle, zoomfx, zoomfy, smooth))!=NULL) {
 				switch (position) {
@@ -112,11 +112,13 @@ void RotatePicture (SDL_Surface *picture, int rotate, int flip, int smooth, int
 				}				
 				SDL_RenderCopy(renderer, rotozoom_texture, NULL, &dest);				
 				SDL_DestroyTexture(rotozoom_texture);
+			} else {
+				SDL_LogError(SDL_LOG_CATEGORY_APPLICATION, "Couldn't rotozoom image\n");
 			}
 		} else {
 			angle=framecount*rotate;
 			if ((framecount % 120)==0) {
-				printf ("  Frame: %i   Rotate: angle=%.2f  Zoom: f=%.2f \n",framecount,angle,zoomf);
+				SDL_Log("  Frame: %i   Rotate: angle=%.2f  Zoom: f=%.2f \n",framecount,angle,zoomf);
 			}
 			if ((rotozoom_picture=rotozoomSurface (picture, angle, zoomf, smooth))!=NULL) {
 				switch (position) {
@@ -141,6 +143,8 @@ void RotatePicture (SDL_Surface *picture, int rotate, int flip, int smooth, int
 				}				
 				SDL_RenderCopy(renderer, rotozoom_texture, NULL, &dest);				
 				SDL_DestroyTexture(rotozoom_texture);
+			} else {
+				SDL_LogError(SDL_LOG_CATEGORY_APPLICATION, "Couldn't rotozoom image\n");
 			}
 		}
 
@@ -176,6 +180,8 @@ void RotatePicture (SDL_Surface *picture, int rotate, int flip, int smooth, int
 				}				
 				SDL_RenderCopy(renderer, rotozoom_texture, NULL, &dest);				
 				SDL_DestroyTexture(rotozoom_texture);
+			} else {
+				SDL_LogError(SDL_LOG_CATEGORY_APPLICATION, "Couldn't rotozoom image\n");
 			}		
 		} else {
 			if ((rotozoom_picture=rotozoomSurface (picture, 0.01, zoomf, smooth))!=NULL) {
@@ -193,6 +199,8 @@ void RotatePicture (SDL_Surface *picture, int rotate, int flip, int smooth, int
 				}				
 				SDL_RenderCopy(renderer, rotozoom_texture, NULL, &dest);				
 				SDL_DestroyTexture(rotozoom_texture);
+			} else {
+				SDL_LogError(SDL_LOG_CATEGORY_APPLICATION, "Couldn't rotozoom image\n");
 			}		
 		}
 
@@ -221,12 +229,12 @@ void ZoomPicture (SDL_Surface *picture, int smooth)
 	SDL_Renderer *renderer = state->renderers[0];
 	SDL_Event event;
 
-	printf("%s\n", messageText);
+	SDL_Log("%s\n", messageText);
 
 	/* Zoom and display the picture */
 	framemax=4*360; frameinc=1;
 	for (framecount=360; framecount<framemax && !done; framecount += frameinc) {
-		while (SDL_PollEvent(&event)) CommonEvent(state, &event, &done);
+		while (SDL_PollEvent(&event)) SDLTest_CommonEvent(state, &event, &done);
 		if ((framecount % 360)==0) frameinc++;
 		SDL_SetRenderDrawColor(renderer, 0xA0, 0xA0, 0xA0, 0xFF);
 		SDL_RenderClear(renderer);
@@ -234,7 +242,7 @@ void ZoomPicture (SDL_Surface *picture, int smooth)
 		zoomxf=1.5*zoomxf*zoomxf;
 		zoomyf=0.5+fabs(1.0*sin((double)framecount/80.0));
 		if ((framecount % 120)==0) {
-			printf ("  Frame: %i   Zoom: x=%.2f y=%.2f\n",framecount,zoomxf,zoomyf);
+			SDL_Log("  Frame: %i   Zoom: x=%.2f y=%.2f\n",framecount,zoomxf,zoomyf);
 		}
 		if ((rotozoom_picture=zoomSurface (picture, zoomxf, zoomyf, smooth))!=NULL) {
 			dest.x = (DEFAULT_WINDOW_WIDTH - rotozoom_picture->w)/2;;
@@ -251,6 +259,8 @@ void ZoomPicture (SDL_Surface *picture, int smooth)
 			}				
 			SDL_RenderCopy(renderer, rotozoom_texture, NULL, &dest);				
 			SDL_DestroyTexture(rotozoom_texture);
+		} else {
+			SDL_LogError(SDL_LOG_CATEGORY_APPLICATION, "Couldn't zoom image\n");
 		}
 
 		stringRGBA(renderer, 8, 8, messageText, 255, 255, 255, 255);
@@ -278,18 +288,18 @@ void RotatePicture90Degrees (SDL_Surface *picture)
 	SDL_Renderer *renderer = state->renderers[0];
 	SDL_Event event;
 
-	printf("%s\n", messageText);
+	SDL_Log("%s\n", messageText);
 
 	/* Rotate and display the picture */
 	framemax = 21;
 	frameinc = 1;
 	numClockwiseTurns = -4;
 	for (framecount=0; framecount<framemax && !done; framecount += frameinc) {
-		while (SDL_PollEvent(&event)) CommonEvent(state, &event, &done);
+		while (SDL_PollEvent(&event)) SDLTest_CommonEvent(state, &event, &done);
 		SDL_SetRenderDrawColor(renderer, 0xA0, 0xA0, 0xA0, 0xFF);
 		SDL_RenderClear(renderer);
-		printf ("  Frame: %i   Rotate90: %i clockwise turns\n",framecount,numClockwiseTurns+4);
-		if ((rotozoom_picture=rotateSurface90Degrees(picture, numClockwiseTurns))!=NULL) {
+		SDL_Log("  Frame: %i   Rotate90: %i clockwise turns\n",framecount,numClockwiseTurns);
+		if ((rotozoom_picture = rotateSurface90Degrees(picture, numClockwiseTurns))!=NULL) {
 			dest.x = (DEFAULT_WINDOW_WIDTH - rotozoom_picture->w)/2;;
 			dest.y = (DEFAULT_WINDOW_HEIGHT - rotozoom_picture->h)/2;
 			dest.w = rotozoom_picture->w;
@@ -304,6 +314,8 @@ void RotatePicture90Degrees (SDL_Surface *picture)
 			}				
 			SDL_RenderCopy(renderer, rotozoom_texture, NULL, &dest);				
 			SDL_DestroyTexture(rotozoom_texture);
+		} else {
+			SDL_LogError(SDL_LOG_CATEGORY_APPLICATION, "Couldn't rotate image: %s\n", SDL_GetError());
 		}
 
 		stringRGBA(renderer, 8, 8, messageText, 255, 255, 255, 255);
@@ -312,7 +324,7 @@ void RotatePicture90Degrees (SDL_Surface *picture)
 		SDL_RenderPresent(renderer);
 
 		/* Always delay */
-		SDL_Delay(333);
+		SDL_Delay(500);
 		if (delay>0) {
 			SDL_Delay(delay);
 		}
@@ -338,8 +350,8 @@ void CustomTest(SDL_Surface *picture, double a, double x, double y, int smooth){
 	SDL_Rect dest;
 	SDL_Renderer *renderer = state->renderers[0];
 
-	printf("%s\n", messageText);
-	printf("  Frame: C   Rotate: angle=%.2f  Zoom: fx=%.2f fy=%.2f \n",a,x,y);
+	SDL_Log("%s\n", messageText);
+	SDL_Log("  Frame: C   Rotate: angle=%.2f  Zoom: fx=%.2f fy=%.2f \n",a,x,y);
 
 	SDL_SetRenderDrawColor(renderer, 0xA0, 0xA0, 0xA0, 0xFF);
 	SDL_RenderClear(renderer);
@@ -358,6 +370,8 @@ void CustomTest(SDL_Surface *picture, double a, double x, double y, int smooth){
 		}				
 		SDL_RenderCopy(renderer, rotozoom_texture, NULL, &dest);				
 		SDL_DestroyTexture(rotozoom_texture);
+	} else {
+		SDL_LogError(SDL_LOG_CATEGORY_APPLICATION, "Couldn't rotozoom image\n");
 	}
 
 	/* Display */
@@ -396,52 +410,62 @@ void Draw (int start, int end)
 	if (start<=6) {
 
 		/* Message */
-		printf("8 bit tests ...\n");
+		SDL_Log("8 bit tests ...\n");
 
 		/* Load the image into a surface */
 		bmpfile = "sample8.bmp";
-		printf("Loading picture: %s\n", bmpfile);
+		SDL_Log("Loading picture: %s\n", bmpfile);
 		picture = SDL_LoadBMP(bmpfile);
 		if ( picture == NULL ) {
-			fprintf(stderr, "Couldn't load %s: %s\n", bmpfile, SDL_GetError());
+			SDL_LogError(SDL_LOG_CATEGORY_APPLICATION,"Couldn't load %s: %s\n", bmpfile, SDL_GetError());
 			return;
 		}
 
 		if (start <= 1) {
-			sprintf(messageText, "1.  rotozoom: Rotating and zooming");
+			SDL_snprintf(messageText, 1024, "1.  rotozoom: Rotating and zooming");
 			RotatePicture(picture,ROTATE_ON,FLIP_OFF,SMOOTHING_OFF,POSITION_CENTER);
+			if (done) goto done8bit;
 		}
 		if (end == 1) goto done8bit;
 
 		if (start <= 2) {
-			sprintf(messageText, "2.  rotozoom: Just zooming (angle=0)");
+			SDL_snprintf(messageText, 1024, "2.a  rotozoom: Just zooming (angle=0, center)");
 			RotatePicture(picture,ROTATE_OFF,FLIP_OFF,SMOOTHING_OFF,POSITION_CENTER);
+			if (done) goto done8bit;
+			SDL_snprintf(messageText, 1024, "2.b  rotozoom: Just zooming (angle=0, bottomright)");
 			RotatePicture(picture,ROTATE_OFF,FLIP_OFF,SMOOTHING_OFF,POSITION_BOTTOMRIGHT);
+			if (done) goto done8bit;
 		}
 		if (end == 2) goto done8bit;
 
 		if (start <= 3) {
-			sprintf(messageText, "3.  zoom: Just zooming");
+			SDL_snprintf(messageText, 1024, "3.  zoom: Just zooming");
 			ZoomPicture(picture,SMOOTHING_OFF);
+			if (done) goto done8bit;
 		}
 		if (end == 3) goto done8bit;
 
 		if (start <= 4) {
-			sprintf(messageText, "4.  rotozoom: Rotating and zooming, interpolation on but unused");
+			SDL_snprintf(messageText, 1024, "4.  rotozoom: Rotating and zooming, interpolation on but unused");
 			RotatePicture(picture,ROTATE_ON,FLIP_OFF,SMOOTHING_ON,POSITION_CENTER);
+			if (done) goto done8bit;
 		}
 		if (end == 4) goto done8bit;
 
 		if (start <= 5) {
-			sprintf(messageText, "5.  rotozoom: Just zooming (angle=0), interpolation on but unused");
+			SDL_snprintf(messageText, 1024, "5.a  rotozoom: Just zooming (angle=0, center), interpolation on but unused");
 			RotatePicture(picture,ROTATE_OFF,FLIP_OFF,SMOOTHING_ON,POSITION_CENTER);
+			if (done) goto done8bit;
+			SDL_snprintf(messageText, 1024, "5.b  rotozoom: Just zooming (angle=0, bottomright), interpolation on but unused");
 			RotatePicture(picture,ROTATE_OFF,FLIP_OFF,SMOOTHING_ON,POSITION_BOTTOMRIGHT);
+			if (done) goto done8bit;
 		}
 		if (end == 5) goto done8bit;
 
 		if (start <= 6) {
-			sprintf(messageText, "6.  zoom: Just zooming, interpolation on but unused");
+			SDL_snprintf(messageText, 1024, "6.  zoom: Just zooming, interpolation on but unused");
 			ZoomPicture(picture,SMOOTHING_ON);
+			if (done) goto done8bit;
 		}
 		if (end == 6) goto done8bit;
 
@@ -449,7 +473,7 @@ done8bit:
 
 		/* Free the picture */
 		SDL_FreeSurface(picture);
-
+		if (done) return;
 		if (end <= 6) return;
 	}
 
@@ -458,14 +482,14 @@ done8bit:
 	if (start<=12) {
 
 		/* Message */
-		printf("24 bit tests ...\n");
+		SDL_Log("24 bit tests ...\n");
 
 		/* Load the image into a surface */
 		bmpfile = "sample24.bmp";
-		printf("Loading picture: %s\n", bmpfile);
+		SDL_Log("Loading picture: %s\n", bmpfile);
 		picture = SDL_LoadBMP(bmpfile);
 		if ( picture == NULL ) {
-			fprintf(stderr, "Couldn't load %s: %s\n", bmpfile, SDL_GetError());
+			SDL_LogError(SDL_LOG_CATEGORY_APPLICATION, "Couldn't load %s: %s\n", bmpfile, SDL_GetError());
 			return;
 		}
 
@@ -473,54 +497,76 @@ done8bit:
 		rectangleColor(renderer, 0, 0, picture->w-1, picture->h-1, 0xffffffff);
 
 		if (start <= 7) {
-			sprintf(messageText, "7.  rotozoom: Rotating and zooming, no interpolation");
+			SDL_snprintf(messageText, 1024, "7.  rotozoom: Rotating and zooming, no interpolation");
 			RotatePicture(picture,ROTATE_ON,FLIP_OFF,SMOOTHING_OFF,POSITION_CENTER);
+			if (done) goto done24bit;			
 		}
 		if (end == 7) goto done24bit;
 
 		if (start <= 8) {
-			sprintf(messageText, "8a.  rotozoom: Just zooming (angle=0), no interpolation, centered");
+			SDL_snprintf(messageText, 1024, "8a.  rotozoom: Just zooming (angle=0), no interpolation, centered");
 			RotatePicture(picture,ROTATE_OFF,FLIP_OFF,SMOOTHING_OFF,POSITION_CENTER);
-			sprintf(messageText, "8b.  rotozoom: Just zooming (angle=0), no interpolation, corner");
+			if (done) goto done24bit;			
+			
+			SDL_snprintf(messageText, 1024, "8b.  rotozoom: Just zooming (angle=0), no interpolation, corner");
 			RotatePicture(picture,ROTATE_OFF,FLIP_OFF,SMOOTHING_OFF,POSITION_BOTTOMRIGHT);
-			sprintf(messageText, "8c.  rotozoom: Just zooming (angle=0), X flip, no interpolation, centered");
+			if (done) goto done24bit;			
+
+			SDL_snprintf(messageText, 1024, "8c.  rotozoom: Just zooming (angle=0), X flip, no interpolation, centered");
 			RotatePicture(picture,ROTATE_OFF,FLIP_X,SMOOTHING_OFF,POSITION_CENTER);
-			sprintf(messageText, "8d.  rotozoom: Just zooming (angle=0), Y flip, no interpolation, centered");
+			if (done) goto done24bit;			
+			
+			SDL_snprintf(messageText, 1024, "8d.  rotozoom: Just zooming (angle=0), Y flip, no interpolation, centered");
 			RotatePicture(picture,ROTATE_OFF,FLIP_Y,SMOOTHING_OFF,POSITION_CENTER);
-			sprintf(messageText, "8e.  rotozoom: Just zooming (angle=0), XY flip, no interpolation, centered");
+			if (done) goto done24bit;			
+						
+			SDL_snprintf(messageText, 1024, "8e.  rotozoom: Just zooming (angle=0), XY flip, no interpolation, centered");
 			RotatePicture(picture,ROTATE_OFF,FLIP_XY,SMOOTHING_OFF,POSITION_CENTER);
+			if (done) goto done24bit;			
 		}
 		if (end == 8) goto done24bit;
 
 		if (start <= 9) {
-			sprintf(messageText, "9.  zoom: Just zooming, no interpolation");
+			SDL_snprintf(messageText, 1024, "9.  zoom: Just zooming, no interpolation");
 			ZoomPicture(picture,SMOOTHING_OFF);
+			if (done) goto done24bit;			
 		}
 		if (end == 9) goto done24bit;
 
 		if (start <= 10) {
-			sprintf(messageText, "10. rotozoom: Rotating and zooming, with interpolation");
+			SDL_snprintf(messageText, 1024, "10. rotozoom: Rotating and zooming, with interpolation");
 			RotatePicture(picture,ROTATE_ON,FLIP_OFF,SMOOTHING_ON,POSITION_CENTER);
+			if (done) goto done24bit;			
 		}
 		if (end == 10) goto done24bit;
 
 		if (start <= 11) {
-			sprintf(messageText, "11a. rotozoom: Just zooming (angle=0), with interpolation, centered");
+			SDL_snprintf(messageText, 1024, "11a. rotozoom: Just zooming (angle=0), with interpolation, centered");
 			RotatePicture(picture,ROTATE_OFF,FLIP_OFF,SMOOTHING_ON,POSITION_CENTER);
-			sprintf(messageText, "11b. rotozoom: Just zooming (angle=0), with interpolation, corner");
+			if (done) goto done24bit;			
+			
+			SDL_snprintf(messageText, 1024, "11b. rotozoom: Just zooming (angle=0), with interpolation, corner");
 			RotatePicture(picture,ROTATE_OFF,FLIP_OFF,SMOOTHING_ON,POSITION_BOTTOMRIGHT);
-			sprintf(messageText, "11c. rotozoom: Just zooming (angle=0), X flip, with interpolation, corner");
+			if (done) goto done24bit;			
+			
+			SDL_snprintf(messageText, 1024, "11c. rotozoom: Just zooming (angle=0), X flip, with interpolation, corner");
 			RotatePicture(picture,ROTATE_OFF,FLIP_X,SMOOTHING_ON,POSITION_CENTER);
-			sprintf(messageText, "11d. rotozoom: Just zooming (angle=0), Y flip, with interpolation, corner");
+			if (done) goto done24bit;			
+			
+			SDL_snprintf(messageText, 1024, "11d. rotozoom: Just zooming (angle=0), Y flip, with interpolation, corner");
 			RotatePicture(picture,ROTATE_OFF,FLIP_Y,SMOOTHING_ON,POSITION_CENTER);
-			sprintf(messageText, "11e. rotozoom: Just zooming (angle=0), XY flip, with interpolation, corner");
+			if (done) goto done24bit;			
+			
+			SDL_snprintf(messageText, 1024, "11e. rotozoom: Just zooming (angle=0), XY flip, with interpolation, corner");
 			RotatePicture(picture,ROTATE_OFF,FLIP_XY,SMOOTHING_ON,POSITION_CENTER);
+			if (done) goto done24bit;			
 		}
 		if (end == 11) goto done24bit;
 
 		if (start <= 12) {
-			sprintf(messageText, "12. zoom: Just zooming, with interpolation");
+			SDL_snprintf(messageText, 1024, "12. zoom: Just zooming, with interpolation");
 			ZoomPicture(picture,SMOOTHING_ON);
+			if (done) goto done24bit;			
 		}
 		if (end == 12) goto done24bit;
 
@@ -528,7 +574,7 @@ done24bit:
 
 		/* Free the picture */
 		SDL_FreeSurface(picture);
-
+                if (done) return;			
 		if (end <= 12) return;
 	}
 
@@ -537,34 +583,38 @@ done24bit:
 	if (start<=16) {
 
 		/* Message */
-		printf("32 bit tests ...\n");
+		SDL_Log("32 bit tests ...\n");
 
 		/* Load the image into a surface */
 		bmpfile = "sample24.bmp";
-		printf("Loading picture: %s\n", bmpfile);
+		SDL_Log("Loading picture: %s\n", bmpfile);
 		picture = SDL_LoadBMP(bmpfile);
 		if ( picture == NULL ) {
-			fprintf(stderr, "Couldn't load %s: %s\n", bmpfile, SDL_GetError());
+			SDL_LogError(SDL_LOG_CATEGORY_APPLICATION, "Couldn't load %s: %s\n", bmpfile, SDL_GetError());
 			return;
 		}
 
 		/* New source surface is 32bit with defined RGBA ordering */
 		/* Much faster to do this once rather than the routine on the fly */
-		fprintf(stderr,"Converting 24bit image into 32bit RGBA surface ...\n");
+		SDL_LogError(SDL_LOG_CATEGORY_APPLICATION,"Converting 24bit image into 32bit RGBA surface ...\n");
 		picture_again = SDL_CreateRGBSurface(SDL_SWSURFACE, picture->w, picture->h, 32, rmask, gmask, bmask, amask);
 		if (picture_again == NULL) goto done32bit;		
 		SDL_BlitSurface(picture,NULL,picture_again,NULL);
 
 		if (start <= 13) {
-			sprintf(messageText, "13. Rotating and zooming, with interpolation (RGBA source)");
+			SDL_snprintf(messageText, 1024, "13. Rotating and zooming, with interpolation (RGBA source)");
 			RotatePicture(picture_again,ROTATE_ON,FLIP_OFF,SMOOTHING_ON,POSITION_CENTER);
+			if (done) goto done32bit;
 		}
 		if (end == 13) goto done32bit;
 
 		if (start <= 14) {
-			sprintf(messageText, "14. Just zooming (angle=0), with interpolation (RGBA source)");
+			SDL_snprintf(messageText, 1024, "14.a Just zooming (angle=0, center), with interpolation (RGBA source)");
 			RotatePicture(picture_again,ROTATE_OFF,FLIP_OFF,SMOOTHING_ON,POSITION_CENTER);
+			if (done) goto done32bit;
+			SDL_snprintf(messageText, 1024, "14.b Just zooming (angle=0, bottomright), with interpolation (RGBA source)");
 			RotatePicture(picture_again,ROTATE_OFF,FLIP_OFF,SMOOTHING_ON,POSITION_BOTTOMRIGHT);
+			if (done) goto done32bit;
 		}
 		if (end == 14) goto done32bit;
 
@@ -573,31 +623,34 @@ done24bit:
 
 		/* New source surface is 32bit with defined ABGR ordering */
 		/* Much faster to do this once rather than the routine on the fly */
-		fprintf(stderr,"Converting 24bit image into 32bit ABGR surface ...\n");
+		SDL_LogError(SDL_LOG_CATEGORY_APPLICATION,"Converting 24bit image into 32bit ABGR surface ...\n");
 		picture_again = SDL_CreateRGBSurface(SDL_SWSURFACE, picture->w, picture->h, 32, amask, bmask, gmask, rmask);
 		if (picture_again == NULL) goto done32bit;		
 		SDL_BlitSurface(picture,NULL,picture_again,NULL);
 
-		if (start <= 14) {
-			sprintf(messageText, "15. Rotating and zooming, with interpolation (ABGR source)");
+		if (start <= 15) {
+			SDL_snprintf(messageText, 1024, "15. Rotating and zooming, with interpolation (ABGR source)");
 			RotatePicture(picture_again,ROTATE_ON,FLIP_OFF,SMOOTHING_ON,POSITION_CENTER);
+			if (done) goto done32bit;
 		}
-		if (end == 14) goto done32bit;
+		if (end == 15) goto done32bit;
 
-		if (start <= 14) {
-			sprintf(messageText, "16. Just zooming (angle=0), with interpolation (ABGR source)");
+		if (start <= 16) {
+			SDL_snprintf(messageText, 1024, "16.a Just zooming (angle=0,center), with interpolation (ABGR source)");
 			RotatePicture(picture_again,ROTATE_OFF,FLIP_OFF,SMOOTHING_ON,POSITION_CENTER);
+			if (done) goto done32bit;
+			SDL_snprintf(messageText, 1024, "16.b Just zooming (angle=0,bottomright), with interpolation (ABGR source)");
 			RotatePicture(picture_again,ROTATE_OFF,FLIP_OFF,SMOOTHING_ON,POSITION_BOTTOMRIGHT);
+			if (done) goto done32bit;
 		}
-		if (end == 14) goto done32bit;
-
+		if (end == 16) goto done32bit;
 
 done32bit:
 
 		/* Free the picture */
 		SDL_FreeSurface(picture);
 		if (picture_again) SDL_FreeSurface(picture_again);
-
+		if (done) return;
 		if (end <= 16) return;
 	}
 
@@ -606,56 +659,62 @@ done32bit:
 	if (start<=22) {
 
 		/* Message */
-		printf("32 bit flip tests ...\n");
+		SDL_Log("32 bit flip tests ...\n");
 
 		/* Load the image into a surface */
 		bmpfile = "sample24.bmp";
-		printf("Loading picture: %s\n", bmpfile);
+		SDL_Log("Loading picture: %s\n", bmpfile);
 		picture = SDL_LoadBMP(bmpfile);
 		if ( picture == NULL ) {
-			fprintf(stderr, "Couldn't load %s: %s\n", bmpfile, SDL_GetError());
+			SDL_LogError(SDL_LOG_CATEGORY_APPLICATION, "Couldn't load %s: %s\n", bmpfile, SDL_GetError());
 			return;
 		}
 
 		/* Excercise flipping functions on 32bit RGBA */
-		printf("Converting 24bit image into 32bit RGBA surface ...\n");
+		SDL_Log("Converting 24bit image into 32bit RGBA surface ...\n");
 		picture_again = SDL_CreateRGBSurface(SDL_SWSURFACE, picture->w, picture->h, 32, rmask, gmask, bmask, amask);
 		if (picture_again == NULL) goto doneflip;
 		SDL_BlitSurface(picture,NULL,picture_again,NULL);
 
 		if (start <= 17) {
-			sprintf(messageText, "17. Rotating with x-flip, no interpolation (RGBA source)");
+			SDL_snprintf(messageText, 1024, "17. Rotating with x-flip, no interpolation (RGBA source)");
 			RotatePicture(picture_again,ROTATE_ON,FLIP_X,SMOOTHING_OFF,POSITION_CENTER);
+			if (done) goto doneflip;
 		}
 		if (end == 17) goto doneflip;
 
 		if (start <= 18) {
-			sprintf(messageText, "18. Rotating with y-flip, no interpolation");
+			SDL_snprintf(messageText, 1024, "18. Rotating with y-flip, no interpolation");
 			RotatePicture(picture_again,ROTATE_ON,FLIP_Y,SMOOTHING_OFF,POSITION_CENTER);
+			if (done) goto doneflip;
 		}
 		if (end == 18) goto doneflip;
 
 		if (start <= 19) {
-			sprintf(messageText, "19. Rotating with xy-flip, no interpolation");
+			SDL_snprintf(messageText, 1024, "19. Rotating with xy-flip, no interpolation");
 			RotatePicture(picture_again,ROTATE_ON,FLIP_XY,SMOOTHING_OFF,POSITION_CENTER);
+			if (done) goto doneflip;
 		}
 		if (end == 19) goto doneflip;
 
 		if (start <= 20) {
-			sprintf(messageText, "20. Rotating with x-flip, with interpolation");
+			SDL_snprintf(messageText, 1024, "20. Rotating with x-flip, with interpolation");
 			RotatePicture(picture_again,ROTATE_ON,FLIP_X,SMOOTHING_ON,POSITION_CENTER);
+			if (done) goto doneflip;
 		}
 		if (end == 20) goto doneflip;
 
 		if (start <= 21) {
-			sprintf(messageText, "21. Rotating with y-flip, with interpolation");
+			SDL_snprintf(messageText, 1024, "21. Rotating with y-flip, with interpolation");
 			RotatePicture(picture_again,ROTATE_ON,FLIP_Y,SMOOTHING_ON,POSITION_CENTER);
+			if (done) goto doneflip;
 		}
 		if (end == 21) goto doneflip;
 
 		if (start <= 22) {
-			sprintf(messageText, "22. Rotating with xy-flip, with interpolation");
+			SDL_snprintf(messageText, 1024, "22. Rotating with xy-flip, with interpolation");
 			RotatePicture(picture_again,ROTATE_ON,FLIP_XY,SMOOTHING_ON,POSITION_CENTER);
+			if (done) goto doneflip;
 		}
 		if (end == 22) goto doneflip;
 
@@ -664,80 +723,83 @@ doneflip:
 		/* Free the pictures */
 		SDL_FreeSurface(picture);
 		if (picture_again) SDL_FreeSurface(picture_again);
-
+		if (done) return;
 		if (end <= 22) return;
 	}
 
 	if (start<=24) {
 
 		/* Message */
-		printf("Loading 24bit image\n");
+		SDL_Log("Loading 24bit image\n");
 
 		/* Load the image into a surface */
 		bmpfile = "sample24.bmp";
-		printf("Loading picture: %s\n", bmpfile);
+		SDL_Log("Loading picture: %s\n", bmpfile);
 		picture = SDL_LoadBMP(bmpfile);
 		if ( picture == NULL ) {
-			fprintf(stderr, "Couldn't load %s: %s\n", bmpfile, SDL_GetError());
+			SDL_LogError(SDL_LOG_CATEGORY_APPLICATION, "Couldn't load %s: %s\n", bmpfile, SDL_GetError());
 			return;
 		}
 
 		/* Excercise flipping functions on 32bit RGBA */
-		fprintf(stderr,"Converting 24bit image into 32bit RGBA surface ...\n");
+		SDL_LogError(SDL_LOG_CATEGORY_APPLICATION,"Converting 24bit image into 32bit RGBA surface ...\n");
 		picture_again = SDL_CreateRGBSurface(SDL_SWSURFACE, picture->w, picture->h, 32, rmask, gmask, bmask, amask);
+		if (picture_again == NULL) goto donecustom32bit;
 		SDL_BlitSurface(picture,NULL,picture_again,NULL);
 
-		sprintf(messageText, "23. CustomTest, values from commandline (32bit)");
+		SDL_snprintf(messageText, 1024, "23. CustomTest, values from commandline (32bit)");
 		CustomTest(picture_again, custom_angle, custom_fx, custom_fy, custom_smooth);
 
-		SDL_FreeSurface(picture_again);
+donecustom32bit:
 
 		/* Free the picture */
 		SDL_FreeSurface(picture);
+		if (picture_again) SDL_FreeSurface(picture_again);
+		if (done) return;
 
 		/* Message */
-		printf("Loading 8bit image\n");
+		SDL_Log("Loading 8bit image\n");
 
 		/* Load the image into a surface */
 		bmpfile = "sample8.bmp";
-		printf("Loading picture: %s\n", bmpfile);
+		SDL_Log("Loading picture: %s\n", bmpfile);
 		picture = SDL_LoadBMP(bmpfile);
-		if ( picture == NULL ) {
-			fprintf(stderr, "Couldn't load %s: %s\n", bmpfile, SDL_GetError());
+		if (picture == NULL) {
+			SDL_LogError(SDL_LOG_CATEGORY_APPLICATION, "Couldn't load %s: %s\n", bmpfile, SDL_GetError());
 			return;
 		}
 
-		sprintf(messageText, "24. CustomTest, values from commandline (8bit)");
+		SDL_snprintf(messageText, 1024, "24. CustomTest, values from commandline (8bit)");
 		CustomTest(picture, custom_angle, custom_fx, custom_fy, custom_smooth);
 
 		/* Free the picture */
 		SDL_FreeSurface(picture);
-
+		if (done) return;
 		if (end <= 24) return;
 	}
 
 	if (start<=25) {
 
 		/* Message */
-		printf("Loading 24bit image\n");
+		SDL_Log("Loading 24bit image\n");
 
 		/* Load the image into a surface */
 		bmpfile = "sample24.bmp";
-		printf("Loading picture: %s\n", bmpfile);
+		SDL_Log("Loading picture: %s\n", bmpfile);
 		picture = SDL_LoadBMP(bmpfile);
 		if ( picture == NULL ) {
-			fprintf(stderr, "Couldn't load %s: %s\n", bmpfile, SDL_GetError());
+			SDL_LogError(SDL_LOG_CATEGORY_APPLICATION, "Couldn't load %s: %s\n", bmpfile, SDL_GetError());
 			return;
 		}
 
 		/* New source surface is 32bit with defined RGBA ordering */
-		printf("Converting 24bit image into 32bit RGBA surface ...\n");
+		SDL_Log("Converting 24bit image into 32bit RGBA surface ...\n");
 		picture_again = SDL_CreateRGBSurface(SDL_SWSURFACE, picture->w, picture->h, 32, rmask, gmask, bmask, amask);
 		if (picture_again == NULL) goto donerotate90;
 		SDL_BlitSurface(picture,NULL,picture_again,NULL);
 
 		/* Excercise rotate90 function on 32bit RGBA */
-		sprintf(messageText, "25.  rotate90: Rotate 90 degrees clockwise (32bit)");
+		SDL_snprintf(messageText, 1024, "25.  rotate90: Rotate 90 degrees clockwise (32bit)");
 		RotatePicture90Degrees(picture_again);
 
 donerotate90:
@@ -745,7 +807,7 @@ donerotate90:
 		/* Free the pictures */
 		SDL_FreeSurface(picture);
 		if (picture_again) SDL_FreeSurface(picture_again);
-
+		if (done) return;
 		if (end <= 25) return;
 	}
 
@@ -757,12 +819,14 @@ donerotate90:
 
 int main(int argc, char *argv[])
 {
-	int i, drawn, test;
+	int i;
+	int testStart = 0;
+	int testEnd = 25;
 	SDL_Event event;
 	Uint32 then, now, frames;
 
 	/* Initialize test framework */
-	state = CommonCreateState(argv, SDL_INIT_VIDEO);
+	state = SDLTest_CommonCreateState(argv, SDL_INIT_VIDEO);
 	if (!state) {
 		return 1;
 	}
@@ -770,22 +834,42 @@ int main(int argc, char *argv[])
 	for (i = 1; i < argc;) {
 		int consumed;
 
-		consumed = CommonArg(state, i);
+		consumed = SDLTest_CommonArg(state, i);
+		if (consumed == 0) {
+                   consumed = -1;
+                   if (SDL_strcasecmp(argv[i], "--start") == 0) {
+                      if (argv[i + 1]) {
+                         testStart = SDL_atoi(argv[i + 1]);
+                         if (testStart < 0) testStart = 0;
+                         consumed = 2;
+                      }
+                   }
+                   else if (SDL_strcasecmp(argv[i], "--end") == 0) {
+                      if (argv[i + 1]) {
+                         testEnd = SDL_atoi(argv[i + 1]);
+                         if (testEnd < 0) testEnd = 0;
+                         consumed = 2;
+                      }
+                   }
+                }
+                
 		if (consumed < 0) {
-			fprintf(stderr,
-				"Usage: %s %s\n",
-				argv[0], CommonUsage(state));
+			SDL_Log("Usage: %s %s [--start #] [--end #]\n",
+				argv[0], SDLTest_CommonUsage(state));
 			return 1;
 		}
 		i += consumed;
 	}
-	if (!CommonInit(state)) {
+	if (!SDLTest_CommonInit(state)) {
 		return 2;
 	}
 
 	/* Create the windows and initialize the renderers */
 	for (i = 0; i < state->num_windows; ++i) {
 		SDL_Renderer *renderer = state->renderers[i];
+                SDL_RendererInfo info;
+                SDL_GetRendererInfo(state->renderers[i], &info);		                
+                SDL_Log("Renderer %i: %s %s", i, info.name, (info.flags | SDL_RENDERER_ACCELERATED) ? "(Accelerated)" : "");		
 		SDL_SetRenderDrawColor(renderer, 0xA0, 0xA0, 0xA0, 0xFF);
 		SDL_RenderClear(renderer);
 	}
@@ -795,65 +879,32 @@ int main(int argc, char *argv[])
 
 	/* Create common message buffer */
 	messageText = (char *)SDL_malloc(1024);
+	if (messageText == NULL) {
+		return -1;
+	}
 
 	/* Main render loop */
 	frames = 0;
 	then = SDL_GetTicks();
 	done = 0;
-	drawn = 0;
-	test = 0;
 	while (!done) {
 		/* Check for events */
 		++frames;
-		while (SDL_PollEvent(&event)) {
-			CommonEvent(state, &event, &done);
-			switch (event.type) {
-			case SDL_KEYDOWN: {
-				switch (event.key.keysym.sym) {
-				case SDLK_SPACE: {
-					/* Switch to next test */
-					test++;
-					drawn = 0;
-					break;
-								 }
-				}
-				break;
-							  }
-			case SDL_MOUSEBUTTONDOWN: {
-				switch (event.button.button) {
-				case SDL_BUTTON_LEFT: {
-					/* Switch to next test */
-					test++;
-					drawn = 0;
-					break;
-									  }
-				case SDL_BUTTON_RIGHT: {
-					/* Switch to prev test */
-					test--;
-					drawn = 0;
-					break;
-									   }
-				}
-				break;
-									  }
-			}
-		}
-
-		if (!drawn) {
-			/* Do all the drawing work */ 
-			Draw(0, 25);
-			drawn = 1;
+		while (SDL_PollEvent(&event) && !done) {
+			SDLTest_CommonEvent(state, &event, &done);
 		}
 
+                /* Do all the drawing work */ 
+                Draw(testStart, testEnd);
 	}
 
-	CommonQuit(state);
+	SDLTest_CommonQuit(state);
 
 	/* Print out some timing information */
 	now = SDL_GetTicks();
 	if (now > then) {
 		double fps = ((double) frames * 1000) / (now - then);
-		printf("%2.2f frames per second\n", fps);
+		SDL_Log("%2.2f frames per second\n", fps);
 	}
 
 	if (messageText) {
